@@ -282,6 +282,9 @@ void bind_torrent_handle()
 #define _ allow_threads
 
     class_<torrent_handle>("torrent_handle")
+        .def(self == self)
+        .def(self != self)
+        .def(self < self)
         .def("get_peer_info", get_peer_info)
         .def("status", _(&torrent_handle::status))
         .def("get_download_queue", get_download_queue)
@@ -319,6 +322,11 @@ void bind_torrent_handle()
         .def("filter_piece", _(&torrent_handle::filter_piece))
         .def("is_piece_filtered", _(&torrent_handle::is_piece_filtered))
         .def("write_resume_data", _(&torrent_handle::write_resume_data))
+        .def("is_seed", _(&torrent_handle::is_seed))
+        .def("is_finished", _(&torrent_handle::is_finished))
+        .def("is_paused", _(&torrent_handle::is_paused))
+        .def("is_auto_managed", _(&torrent_handle::is_auto_managed))
+        .def("has_metadata", _(&torrent_handle::has_metadata))
 #endif
         .def("add_piece", add_piece)
         .def("read_piece", _(&torrent_handle::read_piece))
@@ -335,7 +343,9 @@ void bind_torrent_handle()
         .def("save_resume_data", _(&torrent_handle::save_resume_data))
         .def("force_reannounce", _(force_reannounce0))
         .def("force_reannounce", &force_reannounce)
+#ifndef TORRENT_DISABLE_DHT
         .def("force_dht_announce", _(&torrent_handle::force_dht_announce))
+#endif
         .def("scrape_tracker", _(&torrent_handle::scrape_tracker))
         .def("name", _(&torrent_handle::name))
         .def("set_upload_limit", _(&torrent_handle::set_upload_limit))
