@@ -230,6 +230,9 @@ namespace libtorrent
 		void request_large_blocks(bool b)
 		{ m_request_large_blocks = b; }
 
+		void set_endgame(bool b) { m_endgame_mode = b; }
+		bool endgame() const { return m_endgame_mode; }
+
 		bool no_download() const { return m_no_download; }
 		void no_download(bool b) { m_no_download = b; }
 
@@ -371,7 +374,12 @@ namespace libtorrent
 
 		bool failed() const { return m_failed; }
 
-		int desired_queue_size() const { return m_desired_queue_size; }
+		int desired_queue_size() const
+		{
+			// this peer is in end-game mode we only want
+			// one outstanding request
+			return m_endgame_mode ? 1: m_desired_queue_size;
+		}
 
 		// compares this connection against the given connection
 		// for which one is more eligible for an unchoke.
@@ -500,6 +508,8 @@ namespace libtorrent
 #endif
 		}
 
+		virtual void append_const_send_buffer(char const* buffer, int size);
+
 #ifndef TORRENT_DISABLE_RESOLVE_COUNTRIES	
 		void set_country(char const* c)
 		{
@@ -535,10 +545,13 @@ namespace libtorrent
 		size_type downloaded_since_unchoke() const
 		{ return m_statistics.total_payload_download() - m_downloaded_at_last_unchoke; }
 
-		void setup_receive();
+		enum sync_t { read_async, read_sync };
+		void setup_receive(sync_t sync = read_sync);
 
 	protected:
 
+		size_t try_read(sync_t s, error_code& ec);
+
 		virtual void get_specific_peer_info(peer_info& p) const = 0;
 
 		virtual void write_choke() = 0;
@@ -615,6 +628,8 @@ namespace libtorrent
 			, std::size_t bytes_transferred);
 		void on_receive_data(error_code const& error
 			, std::size_t bytes_transferred);
+		void on_receive_data_nolock(error_code const& error
+			, std::size_t bytes_transferred);
 
 		// this is the limit on the number of outstanding requests
 		// we have to this peer. This is initialized to the settings
@@ -923,6 +938,10 @@ namespace libtorrent
 		// requesting too many pieces while being choked
 		boost::uint8_t m_choke_rejects;
 
+		// counts the number of recursive calls to on_receive_data
+		// used to limit recursion
+		boost::uint8_t m_read_recurse:5;
+
 		// if this is true, the disconnection
 		// timestamp is not updated when the connection
 		// is closed. This means the time until we can
@@ -1011,6 +1030,13 @@ namespace libtorrent
 		// if this is set to true, the client will not
 		// pick any pieces from this peer
 		bool m_no_download:1;
+
+		// this is set to true if the last time we tried to
+		// pick a piece to download, we could only find
+		// blocks that were already requested from other
+		// peers. In this case, we should not try to pick
+		// another piece until the last one we requested is done
+		bool m_endgame_mode:1;
 		
 		template <std::size_t Size>
 		struct handler_storage
@@ -1059,9 +1085,9 @@ namespace libtorrent
 			friend void* asio_handler_allocate(
 			    std::size_t size, allocating_handler<Handler, Size>* ctx)
 			{
-				assert(size <= Size);
+				TORRENT_ASSERT(size <= Size);
 #ifdef TORRENT_DEBUG
-				assert(!ctx->storage.used);
+				TORRENT_ASSERT(!ctx->storage.used);
 				ctx->storage.used = true;
 #endif
 				return &ctx->storage.bytes;
