@@ -68,9 +68,6 @@ namespace libtorrent
 	{
 		INVARIANT_CHECK;
 
-		// we want large blocks as well, so
-		// we can request more bytes at once
-		request_large_blocks(true);
 		prefer_whole_pieces(1);
 
 		// we only want left-over bandwidth
@@ -137,7 +134,11 @@ namespace libtorrent
 			TORRENT_ASSERT(receive_buffer_size < t->block_size());
 			ret.bytes_downloaded = t->block_size() - receive_buffer_size;
 		}
-		ret.block_index = (pr.start + ret.bytes_downloaded) / t->block_size();
+		// this is used to make sure that the block_index stays within
+		// bounds. If the entire piece is downloaded, the block_index
+		// would otherwise point to one past the end
+		int correction = ret.bytes_downloaded ? -1 : 0;
+		ret.block_index = (pr.start + ret.bytes_downloaded + correction) / t->block_size();
 		ret.full_block_bytes = t->block_size();
 		const int last_piece = t->torrent_file().num_pieces() - 1;
 		if (ret.piece_index == last_piece && ret.block_index
@@ -256,6 +257,7 @@ namespace libtorrent
 
 		if (error)
 		{
+			m_statistics.received_bytes(0, bytes_transferred);
 #ifdef TORRENT_VERBOSE_LOGGING
 			(*m_logger) << "*** http_seed_connection error: "
 				<< error.message() << "\n";
@@ -276,6 +278,7 @@ namespace libtorrent
 			TORRENT_ASSERT(!m_requests.empty());
 			if (m_requests.empty())
 			{
+				m_statistics.received_bytes(0, bytes_transferred);
 				disconnect(errors::http_error, 2);
 				return;
 			}
@@ -295,6 +298,7 @@ namespace libtorrent
 
 				if (error)
 				{
+					m_statistics.received_bytes(0, bytes_transferred);
 					disconnect(errors::http_parse_error, 2);
 					return;
 				}
@@ -326,6 +330,7 @@ namespace libtorrent
 						m_ses.m_alerts.post_alert(url_seed_alert(t->get_handle(), url()
 							, error_msg));
 					}
+					m_statistics.received_bytes(0, bytes_transferred);
 					disconnect(errors::http_error, 1);
 					return;
 				}
@@ -345,6 +350,7 @@ namespace libtorrent
 					// this means we got a redirection request
 					// look for the location header
 					std::string location = m_parser.header("location");
+					m_statistics.received_bytes(0, bytes_transferred);
 
 					if (location.empty())
 					{
@@ -374,11 +380,20 @@ namespace libtorrent
 				m_response_left = atol(m_parser.header("content-length").c_str());
 				if (m_response_left == -1)
 				{
+					m_statistics.received_bytes(0, bytes_transferred);
 					// we should not try this server again.
 					t->remove_web_seed(m_url, web_seed_entry::http_seed);
 					disconnect(errors::no_content_length, 2);
 					return;
 				}
+				if (m_response_left != front_request.length)
+				{
+					m_statistics.received_bytes(0, bytes_transferred);
+					// we should not try this server again.
+					t->remove_web_seed(m_url, web_seed_entry::http_seed);
+					disconnect(errors::invalid_range, 2);
+					return;
+				}
 				if (payload > m_response_left) payload = m_response_left;
 				m_body_start = m_parser.body_start();
 				m_response_left -= payload;
@@ -406,6 +421,7 @@ namespace libtorrent
 				(*m_logger) << time_now_string() << ": retrying in " << retry_time << " seconds\n";
 #endif
 
+				m_statistics.received_bytes(0, bytes_transferred);
 				// temporarily unavailable, retry later
 				t->retry_web_seed(m_url, web_seed_entry::http_seed, retry_time);
 				t->remove_web_seed(m_url, web_seed_entry::http_seed);
