@@ -48,7 +48,8 @@ using namespace boost::filesystem;
 using namespace libtorrent;
 
 // proxy: 0=none, 1=socks4, 2=socks5, 3=socks5_pw 4=http 5=http_pw
-void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy, int port)
+void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file
+	, int proxy, int port, char const* protocol, bool url_seed)
 {
 	using namespace libtorrent;
 
@@ -62,7 +63,8 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy, i
 
 	char const* test_name[] = {"no", "SOCKS4", "SOCKS5", "SOCKS5 password", "HTTP", "HTTP password"};
 
-	std::cerr << "  ==== TESTING " << test_name[proxy] << " proxy ====" << std::endl;
+	fprintf(stderr, "\n\n  ==== TESTING %s proxy ==== %s ==== %s ===\n\n\n"
+		, test_name[proxy], protocol, url_seed ? "URL seed" : "HTTP seed");
 	
 	if (proxy)
 	{
@@ -77,8 +79,9 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy, i
 	}
 
 	add_torrent_params p;
-	p.save_path = "./tmp2_web_seed";
 	p.ti = torrent_file;
+	p.save_path = "./tmp2_web_seed";
+	p.storage_mode = storage_mode_compact;
 	torrent_handle th = ses.add_torrent(p);
 
 	std::vector<announce_entry> empty;
@@ -151,7 +154,29 @@ void test_transfer(boost::intrusive_ptr<torrent_info> torrent_file, int proxy, i
 	remove_all("./tmp2_web_seed");
 }
 
-int test_main()
+void save_file(char const* filename, char const* data, int size)
+{
+	error_code ec;
+	file out(filename, file::write_only, ec);
+	TEST_CHECK(!ec);
+	if (ec)
+	{
+		fprintf(stderr, "ERROR opening file '%s': %s\n", filename, ec.message().c_str());
+		return;
+	}
+	file::iovec_t b = { (void*)data, size };
+	out.writev(0, &b, 1, ec);
+	TEST_CHECK(!ec);
+	if (ec)
+	{
+		fprintf(stderr, "ERROR writing file '%s': %s\n", filename, ec.message().c_str());
+		return;
+	}
+
+}
+
+// test_url_seed determines whether to use url-seed or http-seed
+int run_suite(char const* protocol, bool test_url_seed)
 {
 	using namespace libtorrent;
 	using namespace boost::filesystem;
@@ -164,60 +189,90 @@ int test_main()
 		create_directory("./tmp1_web_seed/test_torrent_dir");
 	} catch (std::exception&) {}
 
-	int file_sizes[] =
-	{ 5, 16 - 5, 16, 17, 10, 30, 30, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
-		,1,1,1,1,1,1,13,65,34,75,2,3,4,5,23,9,43,4,43,6, 4};
-
-	char random_data[300000];
-	std::srand(10);
-	for (int i = 0; i != sizeof(file_sizes)/sizeof(file_sizes[0]); ++i)
-	{
-		std::generate(random_data, random_data + sizeof(random_data), &std::rand);
-		char filename[200];
-		snprintf(filename, sizeof(filename), "./tmp1_web_seed/test_torrent_dir/test%d", i);
-		error_code ec;
-		file out(filename, file::write_only, ec);
-		TEST_CHECK(!ec);
-		if (ec)
-		{
-			fprintf(stderr, "ERROR opening file '%s': %s\n", filename, ec.message().c_str());
-			return 1;
-		}
-		file::iovec_t b = { random_data, file_sizes[i]};
-		out.writev(0, &b, 1, ec);
-		TEST_CHECK(!ec);
-		if (ec)
-		{
-			fprintf(stderr, "ERROR writing file '%s': %s\n", filename, ec.message().c_str());
-			return 1;
-		}
-	}
-
 	file_storage fs;
-	add_files(fs, path("./tmp1_web_seed/test_torrent_dir"));
+	int piece_size = 16;
+	if (test_url_seed)
+	{
+		int file_sizes[] =
+		{ 5, 16 - 5, 16, 17, 10, 30, 30, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
+			,1,1,1,1,1,1,13,65,34,75,2,3,4,5,23,9,43,4,43,6, 4};
+
+		char* random_data = (char*)malloc(300000);
+		std::srand(10);
+		for (int i = 0; i != sizeof(file_sizes)/sizeof(file_sizes[0]); ++i)
+		{
+			std::generate(random_data, random_data + 300000, &std::rand);
+			char filename[200];
+			snprintf(filename, sizeof(filename), "./tmp1_web_seed/test_torrent_dir/test%d", i);
+			save_file(filename, random_data, file_sizes[i]);
+		}
+
+		add_files(fs, "./tmp1_web_seed/test_torrent_dir");
+		free(random_data);
+	}
+	else
+	{
+		piece_size = 64 * 1024;
+		char* random_data = (char*)malloc(64 * 1024 * 50);
+		std::srand(10);
+		std::generate(random_data, random_data + 64 * 1024 * 50, &std::rand);
+		save_file("./tmp1_web_seed/seed", random_data, 64 * 1024 * 50);
+		fs.add_file("seed", 64 * 1024 * 50);
+		free(random_data);
+	}
 
 	int port = start_web_server();
 
-	libtorrent::create_torrent t(fs, 16);
+	libtorrent::create_torrent t(fs, piece_size);
 	char tmp[512];
-	snprintf(tmp, sizeof(tmp), "http://127.0.0.1:%d/tmp1_web_seed", port);
-	t.add_url_seed(tmp);
+	if (test_url_seed)
+	{
+		snprintf(tmp, sizeof(tmp), "%s://127.0.0.1:%d/tmp1_web_seed", protocol, port);
+		t.add_url_seed(tmp);
+	}
+	else
+	{
+		snprintf(tmp, sizeof(tmp), "http://127.0.0.1:%d/seed", port);
+		t.add_http_seed(tmp);
+	}
 
-	// calculate the hash for all pieces
-	set_piece_hashes(t, "./tmp1_web_seed");
-	std::vector<char> buf;
 	error_code ec;
+	// calculate the hash for all pieces
+	set_piece_hashes(t, "./tmp1_web_seed", ec);
+
+	if (ec)
+	{
+		fprintf(stderr, "error creating hashes for test torrent: %s\n"
+			, ec.message().c_str());
+		TEST_CHECK(false);
+		return 0;
+	}
+
+	std::vector<char> buf;
 	bencode(std::back_inserter(buf), t.generate());
 	boost::intrusive_ptr<torrent_info> torrent_file(new torrent_info(&buf[0], buf.size(), ec));
 
 	for (int i = 0; i < 6; ++i)
-		test_transfer(torrent_file, i, port);
+		test_transfer(torrent_file, i, port, protocol, test_url_seed);
 	
-	torrent_file->rename_file(0, "./tmp2_web_seed/test_torrent_dir/renamed_test1");
-	test_transfer(torrent_file, 0, port);
+	if (test_url_seed)
+	{
+		torrent_file->rename_file(0, "./tmp2_web_seed/test_torrent_dir/renamed_test1");
+		test_transfer(torrent_file, 0, port, protocol, test_url_seed);
+	}
 
 	stop_web_server();
 	remove_all("./tmp1_web_seed");
 	return 0;
 }
 
+int test_main()
+{
+	int ret = 0;
+	for (int i = 0; i < 2; ++i)
+	{
+		run_suite("http", i);
+	}
+	return ret;
+}
+
